#include "setup.h"
#include "definitions.h"
#include "DisplayChannel.h"
#include <vdr/menu.h>

#include "symbols/audio.xpm"
#include "symbols/dolbydigital.xpm"
#include "symbols/encrypted.xpm"
#include "symbols/radio.xpm"
#include "symbols/recording.xpm"
#include "symbols/teletext.xpm"
#include "symbols/vps.xpm"

#define Gap          5

cBitmap cSkinElchiDisplayChannel::bmVps(vps_xpm);
cBitmap cSkinElchiDisplayChannel::bmTeletext(teletext_xpm);
cBitmap cSkinElchiDisplayChannel::bmRadio(radio_xpm);
cBitmap cSkinElchiDisplayChannel::bmAudio(audio_xpm);
cBitmap cSkinElchiDisplayChannel::bmDolbyDigital(dolbydigital_xpm);
cBitmap cSkinElchiDisplayChannel::bmEncrypted(encrypted_xpm);
cBitmap cSkinElchiDisplayChannel::bmRecording(recording_xpm);

// --- cSkinElchiDisplayChannel --------------------------------------------

cSkinElchiDisplayChannel::cSkinElchiDisplayChannel(bool WithInfo)
{
  present = NULL;
  oldpresentvps = -1;
  lastSeenPresent = -1;
  withInfo = WithInfo;
  lineHeight = font->Height();
  frameColor = Theme.Color(clrBackground);
  message = false;
  if (SkinElchiSetup.logo) {
    logox = 64;
    logoy = 48;
  }
  else
    logox = logoy = 0;

     x0 = 0;
     x1 = SkinElchiSetup.logo ? 16 : 0;
     x2 = x1 + lineHeight / 2;
     x3 = x2 + font->Width("00:00");
     x6 = (logox - x1) + lineHeight / 2 + smallfont->Width("000000__000 00.00 00:00");
     x4 = x3 + lineHeight / 2;
     x5 = x4 + Gap;
     x10 = Setup.OSDWidth;
     x9 = x10 - lineHeight / 2;
     x8 = x9 - bmRecording.Width() - 3 - bmEncrypted.Width() - 3 - bmDolbyDigital.Width() - 3 - bmAudio.Width() - 3 - bmTeletext.Width() - 3 - bmVps.Width() - 3;
     x7 = x9 - lineHeight / 2;
     y0 = 0;
     y1 = SkinElchiSetup.logo ? 16 : 0;
     y2 = y1 + lineHeight / 2;
     y3 = y2 + lineHeight / 2;
     y4 = y3 + 3;
     y5 = y4 + lineHeight / 2;
     y6 = y5 + lineHeight / 2;
     y9 = y6 + (withInfo ? lineHeight * 4 : 0);
     y8 = y9 - lineHeight / 2;
     y7 = y8 - lineHeight / 2;
  if (withInfo) {
     osd = cOsdProvider::NewOsd(Setup.OSDLeft, Setup.OSDTop + (Setup.ChannelInfoPos ? 0 : Setup.OSDHeight - y9));
     tArea Areas[] = { { 0, 0, x10 - 1, y9 - 1, 8 } };
     if (osd->CanHandleAreas(Areas, sizeof(Areas) / sizeof(tArea)) == oeOk) {
    	osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
        if (SkinElchiSetup.logo) {
    	    osd->DrawRectangle(logox, y0, x10 - 1, y1 - 1, clrTransparent);
    	    osd->DrawRectangle(x0, logoy, x1 - 1, y9 - 1, clrTransparent);
    	    }
        }
     else {
        if (SkinElchiSetup.logo) {
    	    tArea Areas[] = { { logox, y1, x10 - 1, logoy - 1, 4 },
			    { x1, logoy, x10 - 1, y9 - 1, 4 },
			    { x0, y0, logox - 1, logoy - 1, 8 },};
    	    osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
        }
	else {
    	    tArea Areas[] = { { x0, y0, x10 - 1, y9 - 1, 4 } };
    	    osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
	}
     }

     if (SkinElchiSetup.logo) {
        osd->DrawRectangle(logox, y1, x2 - 1, y3 - 1, Theme.Color(clrChannelNameBg));
        }
     else {
        osd->DrawRectangle(logox, y1, x2 - 1, y2 - 1, clrTransparent);
        osd->DrawEllipse  (logox, y1, x2 - 1, y2 - 1, Theme.Color(clrChannelDateBg), 2);
        osd->DrawRectangle(logox, y2, x2 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
     }
     osd->DrawRectangle(x2, y1, x6 + lineHeight / 2 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
     osd->DrawRectangle(x6 + lineHeight / 2, y1, x9 - 1, y3 - 1, frameColor);
     osd->DrawEllipse  (x6 + lineHeight / 2, y1, x6 + lineHeight - 1, y3 - 1, Theme.Color(clrChannelDateBg), 5);

     osd->DrawRectangle(x9, y1, x10 - 1, y2 - 1, clrTransparent);
     osd->DrawEllipse  (x9, y1, x10 - 1, y2 - 1, frameColor, 1);
     osd->DrawRectangle(x9, y2, x10 - 1, y3 - 1, frameColor);

     osd->DrawRectangle(logox, y3, x10 - 1, y4 - 1, frameColor);

     if (SkinElchiSetup.logo) {
         osd->DrawRectangle(logox, y4, x2 - 1, logoy - 1, Theme.Color(clrChannelNameBg));
	 osd->DrawRectangle(x1, y4, x2 - 1, y6 - 1, Theme.Color(clrChannelNameBg));
     }
     else {
	 osd->DrawRectangle(logox, y4, x2 - 1, y6 - 1, Theme.Color(clrChannelNameBg));
     }

     osd->DrawRectangle(x2, y4, x8 - lineHeight - 1, y6 - 1, Theme.Color(clrChannelNameBg));
     osd->DrawRectangle(x8 - lineHeight, y4, x10 - 1, y6 - 1, frameColor);
     osd->DrawEllipse  (x8 - lineHeight, y4, x8 - lineHeight / 2 - 1, y6 - 1, Theme.Color(clrChannelNameBg), 5);

     osd->DrawRectangle(x1, y6, x3 - 1, y8 - 1, Theme.Color(clrChannelEpgTimeBg));
     osd->DrawRectangle(x3, y6, x10 - 1, y8 - 1, Theme.Color(clrChannelEpgTitleBg));

     osd->DrawRectangle(x1, y8, x2 - 1, y9 - 1, clrTransparent);
     osd->DrawEllipse  (x1, y8, x2 - 1, y9 - 1, Theme.Color(clrChannelEpgTimeBg), 3);

     osd->DrawRectangle(x2, y8, x3 - 1, y9 - 1, Theme.Color(clrChannelEpgTimeBg));
     osd->DrawRectangle(x3, y8, x9 - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg));

     osd->DrawRectangle(x9, y8, x10 - 1, y9 - 1, clrTransparent);
     osd->DrawEllipse  (x9, y8, x10 - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg), 4);
     }
  else {
     osd = cOsdProvider::NewOsd(Setup.OSDLeft, Setup.OSDTop + (Setup.ChannelInfoPos ? 0 : Setup.OSDHeight - y6));
     if (SkinElchiSetup.logo) {
        tArea Areas[] = { { logox, y1, x10 - 1, logoy - 1, 4 },
	    { x1, logoy, x10 - 1, y9 - 1, 4 },
	    { x0, y0, logox - 1, logoy - 1, 8 },};
    	osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
    	osd->DrawRectangle(logox, y0, x10 - 1, y1 - 1, clrTransparent);
    	osd->DrawRectangle(x0, logoy, x1 - 1, y9 - 1, clrTransparent);
        }
     else {
        tArea Areas[] = { { x0, y0, x10 - 1, y9 - 1, 4 } };
        osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
     }
     if (SkinElchiSetup.logo) {
        osd->DrawRectangle(logox, y1, x2 - 1, y3 - 1, Theme.Color(clrChannelNameBg));
        }
     else {
        osd->DrawRectangle(logox, y1, x2 - 1, y2 - 1, clrTransparent);
        osd->DrawEllipse  (logox, y1, x2 - 1, y2 - 1, Theme.Color(clrChannelDateBg), 2);
        osd->DrawRectangle(logox, y2, x2 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
     }
     osd->DrawRectangle(x2, y1, x6 + lineHeight / 2 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
     osd->DrawRectangle(x6 + lineHeight / 2, y1, x9 - 1, y3 - 1, frameColor);
     osd->DrawEllipse  (x6 + lineHeight / 2, y1, x6 + lineHeight - 1, y3 - 1, Theme.Color(clrChannelDateBg), 5);

     osd->DrawRectangle(x9, y1, x10 - 1, y2 - 1, clrTransparent);
     osd->DrawEllipse  (x9, y1, x10 - 1, y2 - 1, frameColor, 1);
     osd->DrawRectangle(x9, y2, x10 - 1, y3 - 1, frameColor);

     osd->DrawRectangle(logox, y3, x10 - 1, y4 - 1, frameColor);

     if (SkinElchiSetup.logo) {
         osd->DrawRectangle(logox, y4, x2 - 1, logoy - 1, Theme.Color(clrChannelNameBg));
	 osd->DrawRectangle(x1, y4, x2 - 1, y5 - 1, Theme.Color(clrChannelNameBg));
     }
     else {
	 osd->DrawRectangle(logox, y4, x2 - 1, y5 - 1, Theme.Color(clrChannelNameBg));
     }

     osd->DrawRectangle(x1, y5, x2 - 1, y6 - 1, clrTransparent);
     osd->DrawEllipse  (x1, y5, x2 - 1, y6 - 1, Theme.Color(clrChannelNameBg), 3);

     osd->DrawRectangle(x2, y4, x8 - lineHeight - 1, y6 - 1, Theme.Color(clrChannelNameBg));

     osd->DrawRectangle(x8 - lineHeight, y4, x10 - 1, y6 - 1, frameColor);
     osd->DrawEllipse  (x8 - lineHeight, y4, x8 - lineHeight / 2 - 1, y6 - 1, Theme.Color(clrChannelNameBg), 5);

     osd->DrawRectangle(x8 - lineHeight / 2, y4, x10 - 1, y5 - 1, frameColor);
     osd->DrawRectangle(x8 - lineHeight / 2, y5, x9 - 1, y6 - 1, frameColor);

     osd->DrawRectangle(x9, y5, x10 - 1, y6 - 1, clrTransparent);
     osd->DrawEllipse  (x9, y5, x10 - 1, y6 - 1, frameColor, 4);

     }

     if (SkinElchiSetup.logo) {
         osd->DrawRectangle(x0, y0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
         osd->DrawRectangle(logox, y0 + 2, logox + 2 - 1, logoy - 1, frameColor);
         osd->DrawRectangle(x1, logoy, logox + 2 - 1, logoy + 2 - 1, frameColor);
     }
}

cSkinElchiDisplayChannel::~cSkinElchiDisplayChannel()
{
  delete osd;
}

void cSkinElchiDisplayChannel::SetChannel(const cChannel *Channel, int Number)
{
  oldpresentvps = -1;
  osd->DrawRectangle(x8, y4, x9 - 1, y6 - 1, frameColor);
  int x = x9;
  lineHeight = font->Height();
  if (Channel) {
     if (!Channel->GroupSep()) {
	 int d = 3;
         bool rec = cRecordControls::Active();
         x -= bmRecording.Width();
	 osd->DrawBitmap(x, y4 + (y6 - y4 - bmRecording.Height()) / 2, bmRecording, Theme.Color(rec ? clrChannelSymbolRecFg : clrChannelSymbolOff), rec ? Theme.Color(clrChannelSymbolRecBg) : frameColor);
         x -= bmEncrypted.Width() + d;
         osd->DrawBitmap(x, y4 + (y6 - y4 - bmEncrypted.Height()) / 2, bmEncrypted, Theme.Color(Channel->Ca() ? clrChannelSymbolOn : clrChannelSymbolOff), frameColor);
	 x -= bmDolbyDigital.Width() + d;
	 osd->DrawBitmap(x, y4 + (y6 - y4 - bmDolbyDigital.Height()) / 2, bmDolbyDigital, Theme.Color(Channel->Dpid(0) ? clrChannelSymbolOn : clrChannelSymbolOff), frameColor);
         x -= bmAudio.Width() + d;
         osd->DrawBitmap(x, y4 + (y6 - y4 - bmAudio.Height()) / 2, bmAudio, Theme.Color(Channel->Apid(1) ? clrChannelSymbolOn : clrChannelSymbolOff), frameColor);
	 if (Channel->Vpid()) {
    	    x -= bmTeletext.Width() + d;
    	    osd->DrawBitmap(x, y4 + (y6 - y4 - bmTeletext.Height()) / 2, bmTeletext, Theme.Color(Channel->Tpid() ? clrChannelSymbolOn : clrChannelSymbolOff), frameColor);
    	    }
	 else if (Channel->Apid(0)) {
    	    x -= bmRadio.Width() + d;
    	    osd->DrawBitmap(x, y4 + (y6 - y4 - bmRadio.Height()) / 2, bmRadio, Theme.Color(clrChannelSymbolOn), frameColor);
    	    }
	 setchannel_x = x;
         }
     char *s;
     asprintf(&s, "%d", Channel->Number());
     osd->DrawText(logox + lineHeight / 2, y1 + lineOffset, s, Theme.Color(clrChannelNumberFg), Theme.Color(clrChannelDateBg), smallfont, x6 - (logox + lineHeight / 2) - lineHeight);
     free(s);
     osd->DrawText(logox + lineHeight / 2, y4 + lineOffset, Channel->Name(), Theme.Color(clrChannelNameFg), Theme.Color(clrChannelNameBg), smallfont, x8 - (logox + lineHeight / 2) - lineHeight);
     if (SkinElchiSetup.logo)
         if (font->Width(Channel->Name()) != 0 ) {
	 cBitmap bmp(logox, logoy, 8);
	 char *filename;
	 asprintf(&filename, "%s/logos/%s%s", cPlugin::ConfigDirectory("skinelchi"), Channel->Name(),".xpm");
	 if (bmp.LoadXpm(filename) && (bmp.Width() == logox) && (bmp.Height() == logoy))
    	    osd->DrawBitmap(0, 0, bmp);
	 else
	    osd->DrawRectangle(0, 0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
	 free(filename);
	 }
    	 else
	 osd->DrawRectangle(0, 0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
     }
}

void cSkinElchiDisplayChannel::SetEvents(const cEvent *Present, const cEvent *Following)
{
  if (!withInfo)
     return;

  if (present != Present)
  {
     lastSeenPresent = -1;
     present = Present;
  }
  else if ((lastSeenPresent < 0) || (oldpresentvps == -1))
  {
     present = Present;
  }

  osd->DrawRectangle(x1, y6, x3 - 1, y9 - 1, Theme.Color(clrChannelEpgTimeBg));
  osd->DrawRectangle(x1, y8, x2 - 1, y9 - 1, clrTransparent);
  osd->DrawEllipse  (x1, y8, x2 - 1, y9 - 1, Theme.Color(clrChannelEpgTimeBg), 3);
  osd->DrawRectangle(x5, y6, x9 - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg));
  for (int i = 0; i < 2; i++) {
      const cEvent *e = !i ? Present : Following;
      if (e) {
         osd->DrawText(x1 + 4, y6 + 2 * i * lineHeight, e->GetTimeString(), Theme.Color(clrChannelEpgTimeFg), Theme.Color(clrChannelEpgTimeBg), font);
         osd->DrawText(x5 + 2, y6 + 2 * i * lineHeight, e->Title(), Theme.Color(clrChannelEpgTitle), Theme.Color(clrChannelEpgTitleBg), font, x7 - x5 - 2);
         osd->DrawText(x5 + 2, y6 + (2 * i + 1) * lineHeight, e->ShortText(), Theme.Color(clrChannelEpgShortText), Theme.Color(clrChannelEpgTitleBg), smallfont, x7 - x5 - 2);
         }
      }
  lineHeight = font->Height();
  cDevice *Device = cDevice::PrimaryDevice();
  const tTrackId *Track = Device->GetTrack(Device->GetCurrentAudioTrack());
  if (Track) {
    int w = smallfont->Width(Track->description);
    if (w != smallfont->Width("")) {
        osd->DrawRectangle(x6 + lineHeight, y1, x9 - w - 1, y3 - 1, frameColor);
        osd->DrawText(x9 - w, y1 + lineOffset, Track->description, Theme.Color(clrChannelSymbolOn), frameColor, smallfont, w);
    }
    else {
	w = smallfont->Width(Track->language);
	if (w != smallfont->Width("")) {
        osd->DrawRectangle(x6 + lineHeight, y1, x9 - w - 1, y3 - 1, frameColor);
        osd->DrawText(x9 - w, y1 + lineOffset, Track->language, Theme.Color(clrChannelSymbolOn), frameColor, smallfont, w);
	}
	else {
    	    osd->DrawRectangle(x6 + lineHeight, y1, x9 - 1, y3 - 1, frameColor);
	}
    }
  }
}

void cSkinElchiDisplayChannel::SetMessage(eMessageType Type, const char *Text)
{
  if (Text) {
     osd->SaveRegion(logox, y4, x10 - 1, y6 - 1);
     if (withInfo)
        osd->DrawRectangle(logox, y4, x10 - 1, y6 - 1, frameColor);
     osd->DrawText(logox + font->Height() / 2, y4, Text, Theme.Color(clrMessageStatusFg + 2 * Type), Theme.Color(clrMessageStatusBg + 2 * Type), font, x10 - logox, 0, taCenter);
     message = true;
     }
  else {
     osd->RestoreRegion();
     message = false;
     }
}

void cSkinElchiDisplayChannel::Flush(void)
{
  if (withInfo) {
     if (!message) {
        cString date = DayDateTime();
        osd->DrawText(x6 - smallfont->Width(date) + smallfont->Width("0") , y1 + lineOffset, date, Theme.Color(clrChannelDateFg), Theme.Color(clrChannelDateBg), smallfont);
        }

     int seen = 0;
     if (present) {
        time_t t = time(NULL);
        if (t > present->StartTime())
           seen = min(y9 - y6 - 1, int((y9 - y6) * double(t - present->StartTime()) / present->Duration()));

	int presentvps = (int)present->Vps();
    	if((presentvps)!=oldpresentvps) {
	    if ( presentvps != 0) {
		int d = 3;
		int x = setchannel_x;
		x -= bmVps.Width() + d;
		osd->DrawBitmap(x, y4 + (y6 - y4 - bmVps.Height()) / 2, bmVps, Theme.Color(((bool)presentvps != present->StartTime())?clrChannelSymbolOn:clrChannelSymbolOff), frameColor);
    		}
	    }
    	    oldpresentvps = presentvps;
	}
     else oldpresentvps = -1;

     if (seen != lastSeenPresent) {
        osd->DrawRectangle(x3 + Gap, y6 + seen, x3 + Gap + ScrollWidth - 1, y9 - 1, Theme.Color(clrChannelTimebarRest));
        if (seen)
        osd->DrawRectangle(x3 + Gap, y6, x3 + Gap + ScrollWidth - 1, y6 + seen - 1, Theme.Color(clrChannelTimebarSeen));
        lastSeenPresent = seen;
        }
     }
  osd->Flush();
}
