#include "definitions.h"
#include "DisplayMenu.h"

// --- cSkinElchiDisplayMenu -----------------------------------------------

cSkinElchiDisplayMenu::cSkinElchiDisplayMenu(void)
{
  const cFont *font = cFont::GetFont(fontOsd);
  lineHeight = font->Height();
  x0 = 0;
  x1 = Setup.OSDWidth;
  y0 = 0;
  y1 = lineHeight;
  y2 = y1 + lineHeight;
  y5 = Setup.OSDHeight;
  y4 = y5 - lineHeight;
  y3 = y4 - lineHeight;
  osd = cOsdProvider::NewOsd(Setup.OSDLeft, Setup.OSDTop);
  tArea Areas[] = { { x0, y0, x1 - 1, y5 - 1, 4 } };
  if (osd->CanHandleAreas(Areas, sizeof(Areas) / sizeof(tArea)) == oeOk)
     osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
  else {
     tArea Areas[] = { { x0, y0, x1 - 1, y1 - 1, 2 },
                       { x0, y1, x1 - 1, y3 - 1, 2 },
                       { x0, y3, x1 - 1, y5 - 1, 4 }
                     };
     osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
     }
  osd->DrawRectangle(x0, y0, x0 + lineHeight / 2 - 1, y1 - lineHeight / 2 - 1, clrTransparent);
  osd->DrawEllipse(x0, y0, x0 + lineHeight / 2 - 1, y1 - lineHeight / 2 - 1, Theme.Color(clrMenuTitleBg), 2);
  osd->DrawRectangle(x0, y0 + lineHeight / 2, x0 + lineHeight / 2 - 1, y1 - 1, Theme.Color(clrMenuTitleBg));
  osd->DrawRectangle(x1 - lineHeight / 2, y0, x1 - 1, y1 - lineHeight / 2 - 1, clrTransparent);
  osd->DrawEllipse(x1 - lineHeight / 2, y0, x1 - 1, y1 - lineHeight / 2  - 1, Theme.Color(clrMenuTitleBg), 1);
  osd->DrawRectangle(x1 - lineHeight / 2, y0 + lineHeight / 2, x1 - 1, y1 - 1, Theme.Color(clrMenuTitleBg));
  osd->DrawRectangle(x0, y1, x1 - 1, y4 - 1, Theme.Color(clrBackground));
}

cSkinElchiDisplayMenu::~cSkinElchiDisplayMenu()
{
  delete osd;
}

void cSkinElchiDisplayMenu::SetScrollbar(void)
{
  if (textScroller.CanScroll()) {
     int yt = textScroller.Top();
     int yb = yt + textScroller.Height();
     int st = yt;
     int sb = yb;
     int tt = st + (sb - st) * textScroller.Offset() / textScroller.Total();
     int tb = tt + (sb - st) * textScroller.Shown() / textScroller.Total();
     int xl = x1 - ScrollWidth;
     osd->DrawRectangle(xl, st, x1 - 1, sb, Theme.Color(clrMenuScrollbarTotal));
     osd->DrawRectangle(xl, tt, x1 - 1, tb, Theme.Color(clrMenuScrollbarShown));
     }
}

void cSkinElchiDisplayMenu::Scroll(bool Up, bool Page)
{
  cSkinDisplayMenu::Scroll(Up, Page);
  SetScrollbar();
}

int cSkinElchiDisplayMenu::MaxItems(void)
{
  return (y3 - y2) / lineHeight;
}

void cSkinElchiDisplayMenu::Clear(void)
{
  textScroller.Reset();
  osd->DrawRectangle(x0, y1, x1 - 1, y4 - 1, Theme.Color(clrBackground));
}

void cSkinElchiDisplayMenu::SetTitle(const char *Title)
{
  const cFont *smallfont = cFont::GetFont(fontSml);
  const cFont *font = cFont::GetFont(fontOsd);
  osd->DrawText(x0 + lineHeight / 2, y0, Title, Theme.Color(clrMenuTitleFg), Theme.Color(clrMenuTitleBg), smallfont, x1 - x0 - lineHeight);
  osd->DrawRectangle(x0 + lineHeight / 2, y1 - (font->Height() - smallfont->Height()), x1 - lineHeight / 2 - 1, y1 - 1, Theme.Color(clrMenuTitleBg));
}

void cSkinElchiDisplayMenu::SetButtons(const char *Red, const char *Green, const char *Yellow, const char *Blue)
{
  const cFont *smallfont = cFont::GetFont(fontSml);
  const cFont *font = cFont::GetFont(fontOsd);
  int w = x1 - x0 - lineHeight;
  int t0 = x0 + lineHeight / 2;
  int t1 = x0 + w / 4;
  int t2 = x0 + w / 2;
  int t3 = x1 - w / 4 - lineHeight / 2;
  int t4 = x1 - lineHeight / 2;
  osd->DrawRectangle(x0, y4, x1 + lineHeight / 2 - 1, y5 - lineHeight / 2 - 1, Red    ? Theme.Color(clrButtonRedBg) : Theme.Color(clrBackground));
  osd->DrawRectangle(x0, y4 + lineHeight / 2, x0 + lineHeight / 2 - 1, y5 - 1, clrTransparent);
  osd->DrawEllipse(x0, y4 + lineHeight / 2, x0 + lineHeight / 2 - 1, y5 - 1, Red ? Theme.Color(clrButtonRedBg) : Theme.Color(clrBackground), 3);
  osd->DrawText(t0, y4, Red,    Theme.Color(clrButtonRedFg), Red ? Theme.Color(clrButtonRedBg) : Theme.Color(clrBackground), smallfont, t1 - t0, 0, taCenter);
  osd->DrawRectangle(t0, y5 - (font->Height() - smallfont->Height()), t1 - 1, y5 - 1, Red ? Theme.Color(clrButtonRedBg) : Theme.Color(clrBackground));
  osd->DrawText(t1, y4, Green,  Theme.Color(clrButtonGreenFg), Green ? Theme.Color(clrButtonGreenBg) : Theme.Color(clrBackground), smallfont, t2 - t1, 0, taCenter);
  osd->DrawRectangle(t1, y5 - (font->Height() - smallfont->Height()), t2 - 1, y5 - 1, Green ? Theme.Color(clrButtonGreenBg) : Theme.Color(clrBackground));
  osd->DrawText(t2, y4, Yellow, Theme.Color(clrButtonYellowFg), Yellow ? Theme.Color(clrButtonYellowBg) : Theme.Color(clrBackground), smallfont, t3 - t2, 0, taCenter);
  osd->DrawRectangle(t2, y5 - (font->Height() - smallfont->Height()), t3 - 1, y5 - 1, Yellow ? Theme.Color(clrButtonYellowBg) : Theme.Color(clrBackground));
  osd->DrawText(t3, y4, Blue,   Theme.Color(clrButtonBlueFg), Blue ? Theme.Color(clrButtonBlueBg) : Theme.Color(clrBackground), smallfont, t4 - t3, 0, taCenter);
  osd->DrawRectangle(t3, y5 - (font->Height() - smallfont->Height()), t4 - 1, y5 - 1, Blue ? Theme.Color(clrButtonBlueBg) : Theme.Color(clrBackground));
  osd->DrawRectangle(x1 - lineHeight / 2, y4, x1 - 1, y5 - lineHeight / 2 - 1, Blue ? Theme.Color(clrButtonBlueBg) : Theme.Color(clrBackground));
  osd->DrawRectangle(x1 - lineHeight / 2, y4 + lineHeight / 2, x1 - 1, y5 - 1, clrTransparent);
  osd->DrawEllipse(x1 - lineHeight / 2, y4 + lineHeight / 2, x1 - 1, y5 - 1, Blue ? Theme.Color(clrButtonBlueBg) : Theme.Color(clrBackground), 4);
}

void cSkinElchiDisplayMenu::SetMessage(eMessageType Type, const char *Text)
{
  const cFont *font = cFont::GetFont(fontOsd);
  if (Text) {
     osd->DrawRectangle(x0, y3, lineHeight / 2 - 1, y4 - 1, Theme.Color(clrBackground));
     osd->DrawEllipse(x0, y3, lineHeight / 2 - 1, y4 - 1, Theme.Color(clrMessageStatusBg + 2 * Type), 7);
     osd->DrawText(x0 + lineHeight / 2, y3, Text, Theme.Color(clrMessageStatusFg + 2 * Type), Theme.Color(clrMessageStatusBg + 2 * Type), font, x1 - x0 - lineHeight, 0, taCenter);
     osd->DrawRectangle(osd->Width() - lineHeight / 2 - 1, y3, osd->Width() - lineHeight / 2, y4 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
     osd->DrawRectangle(osd->Width() - lineHeight / 2, y3, osd->Width() - 1, y4 - 1, Theme.Color(clrBackground));
     osd->DrawEllipse(osd->Width() - lineHeight / 2, y3, osd->Width() - 1, y4 - 1, Theme.Color(clrMessageStatusBg + 2 * Type), 5);
     }
  else
     osd->DrawRectangle(x0, y3, x1 - 1, y4 - 1, Theme.Color(clrBackground));
}

void cSkinElchiDisplayMenu::SetItem(const char *Text, int Index, bool Current, bool Selectable)
{
  int y = y2 + Index * lineHeight;
  tColor ColorFg, ColorBg;
  if (Current) {
     ColorFg = Theme.Color(clrMenuItemCurrentFg);
     ColorBg = Theme.Color(clrMenuItemCurrentBg);
     }
  else {
     ColorFg = Theme.Color(Selectable ? clrMenuItemSelectable : clrMenuItemNonSelectable);
     ColorBg = Theme.Color(clrBackground);
     }
  const cFont *font = cFont::GetFont(fontOsd);
  osd->DrawEllipse(x0, y, x0 + lineHeight / 2 - 1, y + lineHeight - 1, ColorBg, 7);
  for (int i = 0; i < MaxTabs; i++) {
      const char *s = GetTabbedText(Text, i);
      if (s) {
         int xt = x0 + lineHeight / 2 + Tab(i);
         osd->DrawText(xt, y, s, ColorFg, ColorBg, font, x1 - xt - lineHeight / 2);
         }
      if (!Tab(i + 1))
         break;
      }
  osd->DrawEllipse(x1 - lineHeight/2, y, x1 - 1, y + lineHeight - 1, ColorBg, 5);
  SetEditableWidth(x1 - x0 - Tab(1) - lineHeight / 2);
}

void cSkinElchiDisplayMenu::SetEvent(const cEvent *Event)
{
  if (!Event)
     return;
  const cFont *font = cFont::GetFont(fontOsd);
  int xl = x0 + 10;
  int y = y2;
  cTextScroller ts;
  char t[32];
  snprintf(t, sizeof(t), "%s  %s - %s", *Event->GetDateString(), *Event->GetTimeString(), *Event->GetEndTimeString());
  ts.Set(osd, xl, y, x1 - xl, y3 - y, t, font, Theme.Color(clrMenuEventTime), Theme.Color(clrBackground));
  if (Event->Vps() && Event->Vps() != Event->StartTime()) {
     char *buffer;
     asprintf(&buffer, " VPS: %s", *Event->GetVpsString());
     const cFont *font = cFont::GetFont(fontSml);
     osd->DrawText(x1 - font->Width(buffer) - lineHeight / 2, y, buffer, Theme.Color(clrMenuEventVpsFg), Theme.Color(clrMenuEventVpsBg), font);
     free(buffer);
     }
  y += ts.Height();
  y += font->Height();
  ts.Set(osd, xl, y, x1 - xl, y3 - y, Event->Title(), font, Theme.Color(clrMenuEventTitle), Theme.Color(clrBackground));
  y += ts.Height();
  if (!isempty(Event->ShortText())) {
     const cFont *font = cFont::GetFont(fontSml);
     ts.Set(osd, xl, y, x1 - xl, y3 - y, Event->ShortText(), font, Theme.Color(clrMenuEventShortText), Theme.Color(clrBackground));
     y += ts.Height();
     }
  y += font->Height();
  if (!isempty(Event->Description())) {
     textScroller.Set(osd, xl, y, x1 - xl - 2 * ScrollWidth, y3 - y, Event->Description(), font, Theme.Color(clrMenuEventDescription), Theme.Color(clrBackground));
     SetScrollbar();
     }
}

void cSkinElchiDisplayMenu::SetRecording(const cRecording *Recording)
{
  SetText(Recording->Summary(), false); //TODO
}

void cSkinElchiDisplayMenu::SetText(const char *Text, bool FixedFont)
{
  const cFont *font = cFont::GetFont(FixedFont ? fontFix : fontOsd);
  textScroller.Set(osd, x0 + lineHeight / 2, y2, x1 - x0 - 2 * ScrollWidth - lineHeight, y3 - y2, Text, font, Theme.Color(clrMenuText), Theme.Color(clrBackground));
  SetScrollbar();
}

void cSkinElchiDisplayMenu::Flush(void)
{
  cString date = DayDateTime();
  const cFont *font = cFont::GetFont(fontOsd);
  osd->DrawText(x1 - font->Width(date) - 2 - lineHeight / 2, y0, date, Theme.Color(clrMenuDate), Theme.Color(clrMenuTitleBg), font);
  osd->DrawRectangle(x1 - 2 - lineHeight / 2, y0, x1 - lineHeight / 2, y1 - 1, Theme.Color(clrMenuTitleBg));
  osd->Flush();
}
